@@ -17,6 +17,20 @@ except ImportError:
     PANDAS_SUPPORT = False
 
 
+class ConnectionError(Exception):
+    """
+    Connection Errors
+    """
+    pass
+
+
+class ShoojuApiError(Exception):
+    """
+    Shooju API errors
+    """
+    pass
+
+
 def sid(*args):
     """
     Constructs a series id using the list of arguments
@@ -182,9 +196,6 @@ class Connection(object):
         self._shooju_api = ShoojuApi(server, user, api_key)
         self.job_source = "python"
 
-        # testing connection with server to make sure credentials are valid and server is up
-        self._shooju_api.ping()
-
     @property
     def raw(self):
         """
@@ -226,7 +237,7 @@ class Connection(object):
         """
         self.job_source = name
 
-    def search(self, query='', date_start=None, date_finish=None, fields=None, size=0, query_size=10): #size => points_limit, query_size => series_limit ? and change in API?
+    def search(self, query='', date_start=None, date_finish=None, fields=None, size=10, query_size=10):
         """
         Performs a block search against the Shooju API
 
@@ -242,7 +253,7 @@ class Connection(object):
         data = self._shooju_api.search_series(query, date_start, date_finish, size, query_size, fields)
         return self._process_search_response(data)
 
-    def scroll(self, query='', date_start=None, date_finish=None, fields=None, size=0): #size => points_limit and change in API?
+    def scroll(self, query='', date_start=None, date_finish=None, fields=None, size=10):
         """
         Performs a scroll search. Function is a generator, it returns one series at a time
 
@@ -260,9 +271,9 @@ class Connection(object):
 
         r = self._shooju_api.search_series(query, date_start, date_finish, size, fields=fields, scroll=True)
         scroll_id = r['scroll_id']
-        data = self._process_search_response(r)
 
         while True:
+            data = self._process_search_response(r)
             for series_id, obj in data.iteritems():
                 obj['series_id'] = series_id
                 yield obj
@@ -273,7 +284,8 @@ class Connection(object):
 
     def get_points(self, series_id, date_start=None, date_finish=None, size=10):
         """
-        Retrieves points for a series id. Can select time range.
+        Retrieves points for a series id. Can select time range. If series does not exist it returns
+        None
 
         :param str series_id: series id
         :param datetime.datetime date_start: get points < date
@@ -283,11 +295,12 @@ class Connection(object):
         :rtype: list
         """
         data = self._shooju_api.get_series(series_id, date_start, date_finish, size)
-        return map(lambda p: Point(p[0], p[1]), data['points'])
+        return map(lambda p: Point(p[0], p[1]), data['points']) if data is not None else None
 
     def get_point(self, series_id, dt):
         """
-        Retrieves a point from the series
+        Retrieves a point from the series, if the series does not exist it returns None, if the point
+        does not exist it returns a Point with value of None
 
         :param str series_id: series id
         :param datetime.datetime dt: date of the point
@@ -295,18 +308,24 @@ class Connection(object):
         :rtype: shooju.Point
         """
         points = self.get_points(series_id, dt, dt)
-        return points[0] if len(points) == 1 else None
+        if points is None:
+            return None
+        return points[0] if len(points) == 1 else Point(dt, None)
 
     def get_field(self, series_id, field):
         """
-        Retrieves a field value from the series
+        Retrieves a field value from the series. If the field does not exist or the series does not exist
+        it returns None.
 
         :param str series_id: series id
         :param str field: name of the field
         :return: value of the field
         :rtype: str
         """
-        return self.get_fields(series_id, [field])[field]
+        fields = self.get_fields(series_id, [field])
+        if fields is None:
+            return None
+        return fields.get(field)
 
     def get_fields(self, series_id, fields=None):
         """
@@ -316,8 +335,11 @@ class Connection(object):
         :param str series_id: series id
         :param list fields: list of fields to retrieve
         :return: fields of the series
+        :rtype: dict
         """
         data = self._shooju_api.get_series(series_id, size=0, fields=fields)
+        if data is None:
+            return None
         return data.get('fields', {})
 
     def register_job(self, description, notes="", batch_size=1):
@@ -395,14 +417,14 @@ class Connection(object):
                     if value is not None:
                         result[series_id].setdefault('points', []).append(Point(date, value))
             else:
-                result[series_id]['points'] = []
+                result[series_id][u'points'] = []
 
             fields = {}
             for k, field in enumerate(data['fields']['fields']):
                 value = data['fields']['values'][k][i]
                 if value is not None:
                     fields[field] = value
-            result[series_id]['fields'] = fields
+            result[series_id][u'fields'] = fields
         return dict(result)
 
 
@@ -549,7 +571,13 @@ class ShoojuApi(object):
         if fields is not None:
             params['fields'] = ','.join(fields)
 
-        response = self.client.get(self.API_SERIES_SINGLE, params=params)
+        try:
+            response = self.client.get(self.API_SERIES_SINGLE, params=params)
+        except ShoojuApiError as e:
+            if e.message.startswith('series_not_found'):
+                return None
+            raise e
+
         return response
 
     def bulk_series(self, requests, job_id=None):
@@ -763,16 +791,20 @@ def _check_errors(response):
     return response
 
 
-def _check_bulk_api_error(response):
+def _check_bulk_api_error(responses):
     """
-    Checks bulk API response array
+    Checks bulk API response array, if a bulk response has a series_not_found error it gets removed from the response
     :param dict response: api response
     :raise ConnectionError:
     """
     errors = []
-    for response in response['responses']:
+    for i, response in enumerate(responses['responses']):
         if not response['success']:
-            errors.append(_format_error(response))
+            if response.get('error', '') == 'series_not_found':
+                responses['responses'][i] = None
+            else:
+                errors.append(_format_error(response))
+
     if errors:
         raise ShoojuApiError('\n'.join(errors))
 
@@ -1022,6 +1054,8 @@ class GetBulk(object):
         """
         Converts the data from the API to Points and collects fields
         """
+        if response is None: # series didn't exist
+            return None
         if get_type == "get_points":
             return map(lambda p: Point(p[0], p[1]), response['points'])
         elif get_type == "get_fields" or get_type == "get_field":
@@ -1038,6 +1072,7 @@ class GetBulk(object):
     def _ticket(self):
         return len(self._reqs) - 1
 
+
 def flatten(iterable):
     """
     Flattens a list
@@ -1065,16 +1100,3 @@ def roundrobin(*iterables):
             pending -= 1
             nexts = itertools.cycle(itertools.islice(nexts, pending))
 
-class ConnectionError(Exception):
-    """
-    Connection Errors
-    """
-    pass
-
-class ShoojuApiError(Exception):
-    """
-    Shooju API errors
-    """
-    pass
-
-
